//
//  ItemInformationXMLReader.m
//  BBSApp
//
//  Created by Kiwoon Sung on 7/27/08.
//  Copyright 2008 Columbia University. All rights reserved.
//

#import "ItemInformationXMLReader.h"

static NSUInteger parsedItemInformationCounter;

@implementation ItemInformationXMLReader

@synthesize currentItemInformationObject = _currentItemInformationObject;
@synthesize contentOfCurrentItemInformationProperty = _contentOfCurrentItemInformationProperty;
@synthesize listItemInformationObject = _listItemInformationObject;

// Limit the number of parsed item to 50. Otherwise the application runs very slowly on the device.
#define MAX_ITEMS 50

- (void)parserDidStartDocument:(NSXMLParser *)parser
{
    parsedItemInformationCounter = 0;
}

- (void)parseXMLFileAtURL:(NSURL *)URL parseError:(NSError **)error
{	
    // In this parser, a mutable array for an object should be initialized since the result will be like that.
    // Everytime an entry tag is finished to process, create an instance for the item information and add it
    // to the mutable array list as the future result
    self.listItemInformationObject = [[NSMutableArray alloc] init]; //  initWithArray:MAX_ITEMS];
    
    NSXMLParser *parser = [[NSXMLParser alloc] initWithContentsOfURL:URL];
    // Set self as the delegate of the parser so that it will receive the parser delegate methods callbacks.
    [parser setDelegate:self];
    
    // Depending on the XML document you're parsing, you may want to enable these features of NSXMLParser.
    [parser setShouldProcessNamespaces:NO];
    [parser setShouldReportNamespacePrefixes:NO];
    [parser setShouldResolveExternalEntities:NO];
    
    [parser parse];
    
    NSError *parseError = [parser parserError];
    if (parseError && error) {
        *error = parseError;
    }
    
    [parser release];
    
    // Print the number of total item information added to the internal list
    NSLog(@"total: %d", [self.listItemInformationObject count]);
}


- (void)parseXMLWithData:(NSData *)xmlData parseError:(NSError **)error {
    self.listItemInformationObject = [[NSMutableArray alloc] init]; //  initWithArray:MAX_ITEMS];
    
    NSXMLParser *parser = [[NSXMLParser alloc] initWithData:xmlData];
    // Set self as the delegate of the parser so that it will receive the parser delegate methods callbacks.
    [parser setDelegate:self];
    
    // Depending on the XML document you're parsing, you may want to enable these features of NSXMLParser.
    [parser setShouldProcessNamespaces:NO];
    [parser setShouldReportNamespacePrefixes:NO];
    [parser setShouldResolveExternalEntities:NO];
    
    [parser parse];
    
    NSError *parseError = [parser parserError];
    if (parseError && error) {
        *error = parseError;
    }
    
    [parser release];
    
    // Print the number of total item information added to the internal list
    NSLog(@"total: %d", [self.listItemInformationObject count]);
}


-(NSMutableArray *)getListItemInformation {
    return self.listItemInformationObject;
}


- (ItemInformation *)getCurrentItemInformationObject {
    return self.currentItemInformationObject;
}

- (void)parser:(NSXMLParser *)parser didStartElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName attributes:(NSDictionary *)attributeDict
{
    if (qName) {
        elementName = qName;
    }
    
    // Debug message
//    NSLog(@"elementName = %@", elementName);
//    NSLog(@"qName = %@", qName);

    // If the number of parsed items is greater than MAX_ELEMENTS, abort the parse.
    // Otherwise the application runs very slowly on the device.
    if (parsedItemInformationCounter >= MAX_ITEMS) {
        [parser abortParsing];
    }
    
    if ([elementName isEqualToString:@"bbs"] || [elementName isEqualToString:@"list"]) {
        
        parsedItemInformationCounter++;
        
        // TODO: adding the object described by this XML entity should be handled here.
        // Assign a new object representing the XML entity.
        // Need to handle this part to collect objects in the server item list view controller
        self.currentItemInformationObject = [[ItemInformation alloc] init];
       // [(id)[[UIApplication sharedApplication] delegate] performSelectorOnMainThread:@selector(addToItemInformationList:) withObject:self.currentItemObject waitUntilDone:YES];
        
        // Debugging purpose
        NSLog(@"item information found: elementName[%@], namespaceURI[%@], qName[%@]", elementName, namespaceURI, qName);
        
        return;
    }
        
    if ([elementName isEqualToString:@"entry"]) {
        NSLog(@"entry starts");
        
        //self.listItemInformationObject = [[ItemInformation alloc] init];
    } else if ([elementName isEqualToString:@"fileName"]) {
        // Create a mutable string to hold the contents of the 'field' element.
        // The contents are collected in parser:foundCharacters:.
        NSLog(@"fileName tag starts");
        self.contentOfCurrentItemInformationProperty = [NSMutableString string];
        
    } else if ([elementName isEqualToString:@"itemName"]) {
        // Create a mutable string to hold the contents of the 'field' element.
        // The contents are collected in parser:foundCharacters:.
        NSLog(@"itemName tag starts");
        self.contentOfCurrentItemInformationProperty = [NSMutableString string];
        
    } else if ([elementName isEqualToString:@"shortDescription"]) {
        // Create a mutable string to hold the contents of the 'updated' element.
        NSLog(@"shortDescription tag starts");
        self.contentOfCurrentItemInformationProperty = [NSMutableString string];
    } else {
        // The element isn't one that we care about, so set the property that holds the 
        // character content of the current element to nil. That way, in the parser:foundCharacters:
        // callback, the string that the parser reports will be ignored.
        self.contentOfCurrentItemInformationProperty = nil;
    }
}

- (void)parser:(NSXMLParser *)parser didEndElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName
{     
    if (qName) {
        elementName = qName;
    }
    
    // self.contentOfCurrentItemProperty is filled with the characters collocted in parse:parser:foundCharacters
    // therfore, simply assign the property to the corresponding member of the BBS object
    
    // According to the formation of the original BBS application's xml bbs item,
    // when "entry" tag is found, one field in one BBS item can be generated.
    // Currently, we only have two tags in case of a BBS item: field/value tag.
    // Except the field "itemName" and "shortDescription", value in the field tag will be used as the name of the field.
    // The value in the data tag will be used as the value to be assigned to the above field name.
    
    if ([elementName isEqualToString:@"entry"]) {
        NSLog(@"entry ends");
        //self.currentItemInformationObject.itemName = self.contentOfCurrentItemInformationProperty;
        
        // TODO:
        // Create an instance for the item information entry.
        // Generate a list of the item information entries so that it can be used by server item table controller.
//        ItemInformation *oneItemInformation;
//        oneItemInformation = [[ItemInformation alloc] init];
        [self.listItemInformationObject addObject:self.currentItemInformationObject];
        NSLog(@"one added: itemName: %@", self.currentItemInformationObject.itemName);
        self.currentItemInformationObject = [ItemInformation alloc];
        
    } else if ([elementName isEqualToString:@"fileName"]) {
        //NSLog(@"fileName value picked up");
        self.currentItemInformationObject.fileName = self.contentOfCurrentItemInformationProperty;

        // For Debugging purpose
        NSLog(@"value = %@", self.contentOfCurrentItemInformationProperty);
        
    } else if ([elementName isEqualToString:@"itemName"]) {
        //NSLog(@"itemName value picked up");
        
        // If there's no item name, then fill the member with the default value: "No name"
        if ([self.contentOfCurrentItemInformationProperty isEqualToString:@""]) {
            if (self.currentItemInformationObject.fileName != nil) {
                self.currentItemInformationObject.itemName = self.currentItemInformationObject.fileName;
            } else {
                self.currentItemInformationObject.itemName = @"No name";
            }            
        }
        else {
            self.currentItemInformationObject.itemName = self.contentOfCurrentItemInformationProperty;
        }
        
        // For Debugging purpose
        NSLog(@"value = %@", self.contentOfCurrentItemInformationProperty);
        
    } else if ([elementName isEqualToString:@"shortDescription"]) {
        //NSLog(@"shortDescription value picked up");
        if ([self.contentOfCurrentItemInformationProperty isEqualToString:@""]) {
            self.currentItemInformationObject.shortDescription = @"No description";
        } else {        
            self.currentItemInformationObject.shortDescription = self.contentOfCurrentItemInformationProperty;
        }
        
        // For Debugging purpose
        NSLog(@"value = %@", self.contentOfCurrentItemInformationProperty);
    }
    
}

- (void)parser:(NSXMLParser *)parser foundCharacters:(NSString *)string
{
    if (self.contentOfCurrentItemInformationProperty) {
        // If the current element is one whose content we care about, append 'string'
        // to the property that holds the content of the current element.
        if (string != nil)
            [self.contentOfCurrentItemInformationProperty appendString:string];
    }
}

@end
